#include <string>
#include <sstream>

#include "ChFile.hpp"
#include "io/File.hpp"
#include "io/IOException.hpp"
#include "io/out/Output.hpp"
#include "SyntaxErrorException.hpp"

using namespace std;
using lockdown::io::out::Output;
using namespace lockdown;

void lockdown::ChFile::operator()(const std::string& filestring) {
	string filename;
	try {
		stringstream ss;
		ss<<filestring;
		ss>>filename;
		string message = "Handling " + filename + '\n';
		Output::inst()->write(Output::INFO, message);
	
		lockdown::io::File file(filename);
		
		string tmp;
		while(ss>>tmp) {
			if(tmp == "f:") {
				if(!(ss>>tmp)) {
					string error = "Need parameter to f: \n";
					throw SyntaxErrorException(error);
				}
				file.setFlags(tmp);
			}
			else if(tmp == "u:") {
				if(!(ss>>tmp)) {
					string error = "Need parameter to u: \n";
					throw SyntaxErrorException(error);
				}
				file.setUser(tmp);
			}
			else if(tmp == "g:") {
				if(!(ss>>tmp)) {
					string error = "Need parameter to g: \n";
					throw SyntaxErrorException(error);
				}
				file.setGroup(tmp);
			}
			else if(tmp == "p:") {
				if(!(ss>>tmp)) {
					string error = "Need parameter to p: \n";
					throw SyntaxErrorException(error);
				}
				file.setPermissions(tmp);
			}
			else {
				string error = "Unknown option '" + tmp + "'\n";
				throw SyntaxErrorException(error);
			}
		}
	}
	catch(SyntaxErrorException& e) {
		string error = "Syntax error with an option to " + filename + "\n";
		error += e.getMessage() + "\n";
		Output::inst()->write(Output::ERROR, error);
	}
	catch(io::IOException& e) {
		string error = "Error handling: " + e.getMessage() + "\n";
		Output::inst()->write(Output::ERROR, error);
	}
}
