#include <string>
#include <map>
#include <list>
#include <sstream>
#include <algorithm>
#include <fstream>

#include "Executer.hpp"
#include "Defines.hpp"
#include "io/File.hpp"
#include "io/in/Input.hpp"
#include "io/out/Output.hpp"
#include "modifiers.hpp"
#include "SyntaxErrorException.hpp"
#include "ChFile.hpp"

using namespace std;
using lockdown::io::in::Input;
using lockdown::io::File;
using lockdown::io::out::Output;
using lockdown::ChFile;
using namespace lockdown::modifiers;

void lockdown::Executer::execute() {
	string kern_file;

	try {
		Input::inst()->read(settings);
	}
	catch(...) {
		Output::inst()->write(Output::ERROR, "Error reading the conf file\n");
		exit(-1);
	}
	settingReader = settings.begin();

	for(;settingReader != settings.end(); settingReader++) {
		if((*settingReader).size() == 0) 					//Ignore empty string
			continue;
		
		string::size_type pos = (*settingReader).find('#');						//Ignore comments
		if(pos != string::npos) {
			if(pos == 0)
				continue;
			
			(*settingReader) = (*settingReader).substr(0, pos);			//Remove the comment
		}
		defineProcesser();	//Replace define's with their values
		
		stringstream ss;
		ss<<*settingReader;
		string keyword;
		ss>>keyword;							//Get the keyword
		string next;							//Word right after keyword
		ss>>next;
		if(!ss) {								//All keywords need at least 1 parameter
			string error = "Keyword '"+ keyword+"' missing a parameter\n";
			Output::inst()->write(Output::ERROR, error);
			continue;
		}

		int i = (*settingReader).find(next);
		*settingReader = (*settingReader).substr(i, (*settingReader).size() - i);	//Remove the keyword from input
		
		//We know there is a keyword + at least 1 parameter
		if(keyword == "define") {			//If it is a "define" add it to the list
			define();
			continue;
		}
		if(keyword == "writeText") {
			writeText();
			continue;
		}
		if(keyword.at(0) == '/') {
			files.insert(make_pair(keyword, (*settingReader).substr((*settingReader).find(next))));
			continue;
		}
		if(keyword == "kern_file") {
			kern_file = next;		
			continue;
		}
		if(keyword == "login_class") {
			string next2;
			ss>>next2;		//Find the next parameter
			if(!ss) {
				string error = "Syntax error with: " + *settingReader + "\n";
				Output::inst()->write(Output::ERROR, error);
				continue;
			}			
			loginClass.insert(make_pair(next, (*settingReader).substr((*settingReader).find(next2))));
			continue;
		}
		if(keyword == "mount")  {
			mount.push_back((*settingReader));
			continue;
		}
		if(keyword == "kern") {
			kernel.push_back((*settingReader));
			continue;
		}
		if(keyword == "file") {
			file.push_back((*settingReader));
			continue;
		}
		if(keyword == "root_login") {
			rootLogin(next);
			continue;
		}
		else {
			string error = "Unknown keyword: " + keyword + '\n';
			Output::inst()->write(Output::ERROR, error);
		}			
	}
	
	string message;
	
	if(files.size() > 0) {
		multimap<string, string>::iterator index = files.begin();
		list<string> changes;
		string currentFile = index->first;
		for(; index != files.end(); index++) {
			if(index->first != currentFile) {
				message = "Opdating " + currentFile + "\n";
				Output::inst()->write(Output::INFO, message);
				File f(currentFile);
				list<string> current;
				f.read(current);
				modify(current, changes);
				f.write(current);
				currentFile = index->first;
				changes.clear();
			}
			changes.push_back(index->second);
		}
		message = "Opdating " + currentFile + "\n";
		Output::inst()->write(Output::INFO, message);
		File f(currentFile);
		list<string> current;
		f.read(current);
		modify(current, changes);
		f.write(current);
	}
	
	if(kernel.size() > 0 && kern_file != "") {
		message = "Opdating ";
		message	+= kern_file;
		message	+= "\n";
		Output::inst()->write(Output::INFO, message);
		File f(kern_file);
		list<string> current;
		f.read(current);
		modifyKernel(current, kernel);
		f.write(current);
	}
	
	if(mount.size() > 0) {
		message = "Opdating ";
		message	+= DEFAULT_MOUNT_PATH;
		message	+= "\n";
		Output::inst()->write(Output::INFO, message);
		File f(DEFAULT_MOUNT_PATH);
		list<string> current;
		f.read(current);
		modifyMount(current, mount);
		f.write(current);
	}
	
	if(loginClass.size() > 0) {
		message = "Opdating ";
		message	+= DEFAULT_LOGIN_PATH;
		message	+= "\nRemember to run \'cap_mkdb /etc/login.conf\'\n";
		Output::inst()->write(Output::INFO, message);
		File f(DEFAULT_LOGIN_PATH);
		list<string> current;
		f.read(current);
		modifyLoginClass(current, loginClass);
		f.write(current);
	}
	
	for_each(file.begin(), file.end(), ChFile());
}

void lockdown::Executer::define() {
	stringstream ss;
	ss<<*settingReader;
	string keyword;
	ss>>keyword;
	string value;
	ss>>value;
	if(!ss) {
		string error = "define '"+ keyword+"' missing a parameter\n";
		throw SyntaxErrorException(error);
	}
	defines[keyword] = value;
}

void lockdown::Executer::writeText() {
	stringstream ss;
	ss<<*settingReader;
	string file;
	ss>>file;
	
	list<string> text;
	for(settingReader++; settingReader != settings.end(); settingReader++) {
		if((*settingReader).find("EOT") != string::npos)
			break;
		text.push_back((*settingReader));
	}
	string message = "Opdating " + file + '\n';
	Output::inst()->write(Output::INFO, message);
	File f(file, true);
	f.write(text);
}

void lockdown::Executer::defineProcesser() {	
	stringstream ss;
	ss<<(*settingReader);
	string s;
	map<string, string>::const_iterator finder;
	while(ss>>s)	{
			finder = defines.find(s);
			if(finder != defines.end())
				(*settingReader).replace((*settingReader).find(s), s.size(), finder->second);
	}
}

void lockdown::Executer::rootLogin(const std::string& options) {
	bool console = true;
	bool ttys = true;
	if(options.find("noconsole") != string::npos)
		console = false;
	if(options.find("nottys") != string::npos)
		ttys = false;
	if(options.find("ttys") == string::npos || options.find("console") == string::npos) {
		string error = "Syntax Error at root_login: " + options + '\n';
		Output::inst()->write(Output::ERROR, error);
		return;
	}
		
	string message = "Opdating ";
	message	+= DEFAULT_TTYS_PATH;
	message	+= "\n";
	Output::inst()->write(Output::INFO, message);
	File f(DEFAULT_TTYS_PATH);
	list<string> current;
	f.read(current);
	modifyTTYS(current, console, ttys);
	f.write(current);
}
