#include <string>
#include <exception>

#include <stdlib.h>
#include <unistd.h>	
#include <sys/types.h>

#include "Executer.hpp"
#include "LockdownException.hpp"
#include "Defines.hpp"
#include "io/IOException.hpp"
#include "io/out/Output.hpp"
#include "io/out/Stdout.hpp"
#include "io/out/Syslog.hpp"
#include "io/in/Input.hpp"
#include "io/in/File.hpp"

using namespace std;
using lockdown::io::out::Output;
using lockdown::io::in::Input;
using lockdown::LockdownException;
using lockdown::io::IOException;
using lockdown::io::File;

int main(int argc, char **argv) { 
	try {
		for(int i = 1; i < argc; i++) {						//Read program arguments
			string tmp = argv[i];
			if(tmp == "-d") {								//Become a daemon
				Output::inst()->useSyslog(PROGRAMNAME);
				if(daemon(0, 0) == -1) {
					Output::inst()->write(Output::ERROR, "Error calling daemon()");
					exit(-1);
				}
			}
			
			else if(tmp == "-v") {							//Be verbose
				Output::inst()->verbose(true);
			}
			
			else if(tmp == "-f") {							//Specify conf file
				if(i < (argc - 1)) {
					i++;
					string confFile = argv[i];
					Input::inst()->useFile(confFile);	
				}
				else {
					Output::inst()->write(Output::ERROR, "A filename must follow the '-f' flag\n");
					return(-1);
				}
			}
			
			else {											//Unknown argument
				string error = "Sorry, I have no idea what you mean by: " + tmp + "\n";
				Output::inst()->write(Output::ERROR, error);
				exit(-1);
			}
		}	
		
		//Arguments all fine, let's move on
		
		if(getuid() != 0) {
			Output::inst()->write(Output::ERROR, "You must be root to run this program\n");
			exit(-1);
		}
		
		lockdown::Executer::inst()->execute();
		
	}//End try
	catch(File::NotFoundError& e) {
		string error = "Could not find: " + e.getMessage() + "\n";
		Output::inst()->write(Output::ERROR, error);
		return(-1);
	}
	catch(File::OpenError& e) {
		string error = "Could not open: " + e.getMessage() + "\n";
		Output::inst()->write(Output::ERROR, error);
		return(-1);
	}
	catch(IOException& e) {
		string error = "IOException: " + e.getMessage() + "\n";
		Output::inst()->write(Output::ERROR, error);
		return(-1);
	}
	catch(LockdownException& e) {
		string error ="LockdownException: " + e.getMessage() + "\n";
		Output::inst()->write(Output::ERROR, error);
		return(-1);
	}
	catch(exception& e) {
		string error = "Standard exception: ";
		error += e.what();
		error += "\n";
		Output::inst()->write(Output::ERROR, error);
		return(-1);
	}
	catch(...) {
		Output::inst()->write(Output::ERROR, "Unknown exception\n");
		return(-1);
	}
} //End main
