/*	'modifyLoginClass' is for adding options
		to the login.conf file
*/

#include <list>
#include <map>
#include <string>
#include <sstream>

#include "../SyntaxErrorException.hpp"
#include "../io/out/Output.hpp"
#include "../modifiers.hpp"

using namespace std;
using lockdown::io::out::Output;

void lockdown::modifiers::modifyLoginClass(listS& current, std::multimap<std::string, std::string>& changes) {
	listS::iterator currentReader;
	typedef multimap<string, string>::iterator index;
	pair<index, index> lc;
	
	string currentClass;
	
	for(currentReader = current.begin();currentReader != current.end();currentReader++) {
		if(*currentReader == "" || (*currentReader).at(0) == '#')	//Remove comments and empty lines
			continue;
		
		stringstream tt;
		tt<<*currentReader;
		string option;
		tt>>option;

		if(option.at(0) != ':')	{	//It's a new loginclass
			currentClass = option.substr(0, option.size() - 2);
			lc = changes.equal_range(currentClass);
			continue;
		}

		bool lastOption = false;
		string::size_type pos = (*currentReader).find_last_of(':');
		if(pos == string::npos) {
			string error = "Login.conf syntax error with " + option + '\n';
			throw SyntaxErrorException(error);
		}
		
		if(pos == ((*currentReader).size() -1)){		//Is it the last option in the class?
			lastOption = true;
		}
		
		pos = option.find('=');
		if(pos != string::npos) {
			option.erase(pos, option.size() - pos);
		}
		else {
			if(!lastOption)
				option.erase(option.size() - 2, 2);
			else
				option.erase(option.size() - 1, 1);
		}
		
		option.erase(0, 1);		//Remove the first :
		
		for(index a = lc.first;a != lc.second;a++) {	//Does the user want to modify that option?
			string option2 = a->second;
			pos = option2.find('=');
			if(pos != string::npos) 
				option2.erase(pos, option2.size() - pos);
			
			if(option == option2) {
				*currentReader = "\t:" + a->second + ":";
				if(!lastOption)
					*currentReader += "\\";
				if(a == lc.first) {
					changes.erase(a);
					lc = changes.equal_range(currentClass);
				}
				else
					changes.erase(a);
				break;
			}
			if(lastOption) {
				for(index a = lc.first;a != lc.second;) {
					string op = "\t:" + a->second + ":\\";	//Add options not already in the file
					current.insert(currentReader, op);
					index b = a;
					a++;
					changes.erase(b);
				}
				break;
			}
		}
	}
	
	// changes now contains login classes not in the login.conf file
	currentClass = "";
	for(index a = changes.begin();a != changes.end();a++) {
		string newClass = a->first;
		if(newClass != currentClass) {
			currentClass = newClass;
			current.push_back(newClass + ":");
			string message = "Add new login class: ";
			message	+= currentClass;
			message	+= "\n";
			Output::inst()->write(Output::INFO, message);
		}
		current.back() += "\\";
		string tmp =	"\t:" + a->second + ":";
		current.push_back(tmp);
	}
}
